﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Reflection;

namespace DragDataGridView
{
    public partial class Form1 : Form
    {
        private List<Animal> personList;
        private List<Animal> personListForWPF;
        private int indexOfDragRow = -1;
        private int indexOfInsertRow = -1;
        private int indexOfOverRow = -1;
        private Rectangle shieldBox = Rectangle.Empty;
        private TipPanel tipPanel;

        public Form1()
        {
            InitializeComponent();
            this.personList = new List<Animal>();
            this.personListForWPF = new List<Animal>();
            this.tipPanel = new TipPanel();
            this.tipPanel.Visible = false;
            this.dgvAnimalInfo.Controls.Add(this.tipPanel);
            this.tipPanel.BringToFront();
            this.ucWPFDateGrid.OnDragDropCompleted += new OnDragDropCompletedHandler(ucWPF_OnDragDropCompleted);
            this.Load += new EventHandler(Form1_Load);
        }

        void ucWPF_OnDragDropCompleted(object sender, DragDataGridDragCompletedEventArgs e)
        {
            this.lbResult.Items.Add("Control: " + this.ucWPFDateGrid.dgWPFAnimalInfo.Name + "\tOperation: " + e.DragEffect.ToString() + "\t List Count: " + this.personListForWPF.Count.ToString());
        }

        void Form1_Load(object sender, EventArgs e)
        {
            this.personList.Add(new Animal { ID = 1, Name = "WinForm - Tiger", Address = "China" });
            this.personList.Add(new Animal { ID = 2, Name = "WinForm - Snake", Address = "China" });
            this.personList.Add(new Animal { ID = 3, Name = "WinForm - Lion", Address = "China" });
            this.personList.Add(new Animal { ID = 4, Name = "WinForm - Dog", Address = "China" });
            this.personList.Add(new Animal { ID = 5, Name = "WinForm - Cat", Address = "China" });
            this.personList.Add(new Animal { ID = 6, Name = "WinForm - Bird", Address = "China" });
            this.personList.Add(new Animal { ID = 7, Name = "WinForm - Fish", Address = "China" });
            this.personListForWPF.Add(new Animal { ID = 1, Name = "WPF - Tiger", Address = "China" });
            this.personListForWPF.Add(new Animal { ID = 2, Name = "WPF - Snake", Address = "China" });
            this.personListForWPF.Add(new Animal { ID = 3, Name = "WPF - Lion", Address = "China" });
            this.personListForWPF.Add(new Animal { ID = 4, Name = "WPF - Dog", Address = "China" });
            this.personListForWPF.Add(new Animal { ID = 5, Name = "WPF - Cat", Address = "China" });
            this.personListForWPF.Add(new Animal { ID = 6, Name = "WPF - Bird", Address = "China" });
            this.personListForWPF.Add(new Animal { ID = 7, Name = "WPF - Fish", Address = "China" });
            this.dgvAnimalInfo.DataSource = this.personList;
            this.ucWPFDateGrid.DataSource = this.personListForWPF;
        }

        #region WinForm Mode

        private void dgvAnimal_MouseDown(object sender, MouseEventArgs e)
        {
            var hitTest = dgvAnimalInfo.HitTest(e.X, e.Y);
            if (hitTest.Type != DataGridViewHitTestType.Cell)
                return;
            
            this.tipPanel.Location = this.dgvAnimalInfo.PointToClient(e.Location);
            this.tipPanel.Visible = true;
            indexOfDragRow = hitTest.RowIndex;
            if (indexOfDragRow > -1)
            {
                Size dragSize = SystemInformation.DragSize;
                shieldBox = new Rectangle(new Point(e.X - (dragSize.Width / 2), e.Y - (dragSize.Height / 2)), dragSize);
            }
            else
                shieldBox = Rectangle.Empty; 
        }

        private void dgvAnimal_MouseUp(object sender, MouseEventArgs e)
        {
            shieldBox = Rectangle.Empty;
            if (this.tipPanel.Visible)
                this.tipPanel.Visible = false;
        }
        
        private void dgvAnimal_MouseMove(object sender, MouseEventArgs e)
        {            
            if ((e.Button & MouseButtons.Left) != MouseButtons.Left)
                return;
            if (shieldBox == Rectangle.Empty || shieldBox.Contains(e.X, e.Y))
                return;            
            if (indexOfDragRow < 0)
                return;

            var row = dgvAnimalInfo.Rows[indexOfDragRow];
            string strRow = DataSerializer.SerializeObject(row.DataBoundItem as Animal);
            DragDropEffects dropEffect = dgvAnimalInfo.DoDragDrop(strRow, DragDropEffects.All);

            OnRowDragOver(-1); 
        }

        private void dgvAnimal_DragOver(object sender, DragEventArgs e)
        {            
            Point p = dgvAnimalInfo.PointToClient(new Point(e.X, e.Y));
            var hitTest = dgvAnimalInfo.HitTest(p.X, p.Y);
            if (hitTest.Type != DataGridViewHitTestType.Cell || hitTest.RowIndex == indexOfDragRow)
            {
                e.Effect = DragDropEffects.None;
                this.tipPanel.Visible = false;
                OnRowDragOver(-1);
                return;
            }
            e.Effect = e.KeyState.Equals(9/* 9 means Ctrl Key */) ?
                DragDropEffects.Copy :
                DragDropEffects.Move;

            if (this.dgvAnimalInfo.SelectedRows[0].DataBoundItem != null)
            {
                if (!tipPanel.Visible)
                    tipPanel.Visible = true;
                this.tipPanel.Location = p;
                this.tipPanel.ImgInfo = e.Effect.Equals(DragDropEffects.Copy) ? Properties.Resources.ImgCopyMode : Properties.Resources.ImgMoveMode;
                this.tipPanel.InfoText = (this.dgvAnimalInfo.SelectedRows[0].DataBoundItem as Animal).Name;
            }
            
            OnRowDragOver(hitTest.RowIndex); 
        }

        private void dgvAnimal_DragDrop(object sender, DragEventArgs e)
        {
            Point p = dgvAnimalInfo.PointToClient(new Point(e.X, e.Y));
            string strRow = (string)e.Data.GetData(DataFormats.StringFormat);
            Animal aimAnimal = (Animal)DataSerializer.DeserializeObject(strRow);
            Animal tempModel = new Animal();
            this.CopyModel<Animal>(aimAnimal, tempModel);
            this.tipPanel.Visible = false;
            var hitTest = dgvAnimalInfo.HitTest(p.X, p.Y);
            if (hitTest.Type != DataGridViewHitTestType.Cell || hitTest.RowIndex == indexOfDragRow + 1)
                return;

            indexOfInsertRow = hitTest.RowIndex;
            if(e.Effect.Equals(DragDropEffects.Move))
                this.personList.RemoveAt(indexOfDragRow);

            if (indexOfDragRow < indexOfInsertRow)
                indexOfInsertRow--;
            this.personList.Insert(indexOfInsertRow, tempModel);

            this.dgvAnimalInfo.DataSource = null;
            this.dgvAnimalInfo.DataSource = this.personList;
            dgvAnimalInfo.CurrentCell = this.dgvAnimalInfo.Rows[indexOfInsertRow].Cells[0];

            //Debug
            this.lbResult.Items.Add("Control: " + this.dgvAnimalInfo.Name + "\tOperation: " + e.Effect.ToString() + "\t List Count: " + this.personList.Count.ToString());
        }

        private void dgvAnimal_RowPostPaint(object sender, DataGridViewRowPostPaintEventArgs e)
        {
            if (e.RowIndex == indexOfOverRow)
            {
                e.Graphics.FillRectangle
                    (
                        Brushes.DarkGray, 
                        e.RowBounds.X, 
                        e.RowBounds.Y, 
                        this.dgvAnimalInfo.Columns.GetColumnsWidth(DataGridViewElementStates.Visible), 
                        2
                    ); 
            }
        }

        private void OnRowDragOver(int rowIndex)
        {
            if (indexOfOverRow == rowIndex)
                return;
            int oldIndex = indexOfOverRow;
            indexOfOverRow = rowIndex;
            if (oldIndex > -1)
                dgvAnimalInfo.InvalidateRow(oldIndex);
            if (rowIndex > -1)
                dgvAnimalInfo.InvalidateRow(rowIndex);
        }

        #endregion

        //Deep Copy
        private void CopyModel<T>(T oriEntity, T newEntity)
        {
            if (oriEntity == null || newEntity == null)
                return;
            Type type = typeof(T);
            PropertyInfo[] proInfoArray = type.GetProperties();
            foreach (PropertyInfo proInfo in proInfoArray)
            {
                if (proInfo.CanWrite)
                {
                    object oriObj = type.GetProperty(proInfo.Name).GetValue(oriEntity, null);
                    type.GetProperty(proInfo.Name).SetValue(newEntity, oriObj, null);
                }
            }
        }
    }
}
